/*
Copyright (c) 1992-2007 The University of Tennessee.  All rights reserved.

$COPYRIGHT$

Additional copyrights may follow

$HEADER$

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
 met:

- Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer. 
  
- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer listed
  in this license in the documentation and/or other materials
  provided with the distribution.
  
- Neither the name of the copyright holders nor the names of its
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.
  
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT  
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/
/*
Copyright (C) 2008 Nakata, Maho
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/*
Based on http://www.netlib.org/blas/dspr.f
Rspr performs the symmetric rank 1 operation
 A := alpha*x*x' + A,
where alpha is a real scalar, x is an n element vector and A is an
n by n symmetric matrix, supplied in packed form.
*/

#include <mblas.h>

void
Rspr(const char *uplo, int n, mpf_class alpha, mpf_class * x, int incx,
    mpf_class * AP)
{
    int ix, jx, kx, kk;
    mpf_class temp;
    mpf_class Zero = 0.0;

//Test the input parameters.
    int info = 0;

    if (!Mlsame(uplo, "U") && !Mlsame(uplo, "L"))
	info = 1;
    else if (n < 0)
	info = 2;
    else if (incx == 0)
	info = 5;
    if (info != 0) {
	Mxerbla("Rspr  ", info);
	return;
    }
    //quick return if possible.
    if ((n == 0) || (alpha == Zero))
	return;

    //set the start point in x if the increment is not unity.
    if (incx <= 0)
	kx = -(n - 1) * incx;
    else
	kx = 0;

    //start the operations. in this version the elements of the array ap
    //are accessed sequentially with one pass through ap.
    kk = 0;
    if (Mlsame(uplo, "U")) {
	//Form  A  when upper triangle is stored in AP.
	jx = kx;
	for (int j = 0; j < n; j++) {
	    if (x[jx] != Zero) {
		temp = alpha * x[jx];
		ix = kx;
		for (int k = kk; k < kk + j + 1; k++) {
		    AP[k] = AP[k] + x[ix] * temp;
		    ix = ix + incx;
		}
	    }
	    jx = jx + incx;
	    kk = kk + j + 1;
	}
    } else {
	//Form  A  when lower triangle is stored in AP.
	jx = kx;
	for (int j = 0; j < n; j++) {
	    if (x[jx] != Zero) {
		temp = alpha * x[jx];
		ix = jx;
		for (int k = kk; k < kk + n - j; k++) {
		    AP[k] = AP[k] + x[ix] * temp;
		    ix = ix + incx;
		}
	    }
	    jx = jx + incx;
	    kk = kk + n - j;
	}
    }
}
